==> cfn-config-2.2.1.tgz: package/lib/actions.js <==
var url = require('url');
var crypto = require('crypto');
var stream = require('stream');
var AWS = require('aws-sdk');
var cuid = require('cuid');
var error = require('fasterror');
var s3urls = require('s3urls');
var eventStream = require('cfn-stack-event-stream');
var lookup = require('./lookup');

require('colors');

var colors = {
  CREATE_IN_PROGRESS: 'yellow',
  CREATE_FAILED: 'red',
  CREATE_COMPLETE: 'green',
  DELETE_IN_PROGRESS: 'yellow',
  DELETE_FAILED: 'red',
  DELETE_COMPLETE: 'grey',
  UPDATE_IN_PROGRESS: 'yellow',
  UPDATE_COMPLETE_CLEANUP_IN_PROGRESS: 'yellow',
  UPDATE_FAILED: 'red',
  UPDATE_COMPLETE: 'green',
  ROLLBACK_IN_PROGRESS: 'red',
  ROLLBACK_COMPLETE: 'red'
};

var actions = module.exports = {};

/**
 * Create a new CloudFormation stack
 *
 * @param {string} name - the name of the new stack
 * @param {string} region - the region to launch the stack into
 * @param {string} templateUrl - the URL for the template on S3
 * @param {object} parameters - name/value pairs defining the desired stack configuration
 * @param {function} callback - a function called when stack creation is complete
 */
actions.create = function(name, region, templateUrl, parameters, callback) {
  var cfn = new AWS.CloudFormation({ region: region });
  var params = stackParameters(name, templateUrl, parameters);

  cfn.createStack(params, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    callback();
  });
};

/**
 * Update an existing CloudFormation stack
 *
 * @param {string} name - the name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {string} templateUrl - the URL for the template on S3
 * @param {object} parameters - name/value pairs defining the desired stack configuration
 * @param {function} callback - a function called when the stack update has been invoked
 */
actions.update = function(name, region, templateUrl, parameters, callback) {
  var cfn = new AWS.CloudFormation({ region: region });
  var params = stackParameters(name, templateUrl, parameters);

  cfn.updateStack(params, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    callback();
  });
};

/**
 * Determine what will change about an existing CloudFormation stack by
 * performing a specific update
 *
 * @param {string} name - the name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {string} templateUrl - the URL for the template on S3
 * @param {object} parameters - name/value pairs defining the desired stack configuration
 * @param {function} callback - a function provided with a summary of the changes that
 * this change will perform to stack resources
 */
actions.diff = function(name, region, templateUrl, parameters, callback) {
  var cfn = new AWS.CloudFormation({ region: region });
  var changesetId = 'a' + crypto.randomBytes(16).toString('hex');
  var params = stackParameters(name, templateUrl, parameters);
  params.ChangeSetName = changesetId;

  cfn.createChangeSet(params, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));

    describeChangeset(cfn, name, changesetId, function(err, data) {
      if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));

      var details = {
        id: data.ChangeSetName,
        status: data.Status,
        execution: data.ExecutionStatus
      };

      if (data.Changes) details.changes = data.Changes.map(function(change) {
        return {
          id: change.ResourceChange.PhysicalResourceId,
          name: change.ResourceChange.LogicalResourceId,
          type: change.ResourceChange.ResourceType,
          action: change.ResourceChange.Action,
          replacement: change.ResourceChange.Replacement === 'True'
        };
      });

      callback(null, details);
    });
  });
};

/**
 * Execute a ChangeSet in order to perform an update on an existing CloudFormation stack
 *
 * @param {string} name - the name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {string} changesetId - the name or ARN of an existing changeset
 * @param {function} callback - a function called when the stack update has been invoked
 */
actions.executeChangeSet = function(name, region, changesetId, callback) {
  var cfn = new AWS.CloudFormation({ region: region });

  describeChangeset(cfn, name, changesetId, function(err, data) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));

    if (data.ExecutionStatus !== 'AVAILABLE') {
      err = new actions.ChangeSetNotExecutableError('Cannot execute changeset');
      err.execution = data.ExecutionStatus;
      err.status = data.Status;
      err.reason = data.StatusReason;
      return callback(err);
    }

    cfn.executeChangeSet({ StackName: name, ChangeSetName: changesetId }, function(err) {
      if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
      callback();
    });
  });
};

/**
 * Delete an existing CloudFormation stack
 *
 * @param {string} name - the name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {function} callback - a function called when stack deletion is complete
 */
actions.delete = function(name, region, callback) {
  var cfn = new AWS.CloudFormation({ region: region });
  cfn.deleteStack({ StackName: name }, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    callback();
  });
};

/**
 * Monitor a stack throughout a create, delete, or update
 *
 * @param {string} name - the full name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {number} [pollInterval=10000] - ms to wait between requests for
 * CloudFormation stack events
 * @param {function} callback - a function called when stack action is complete
 */
actions.monitor = function(name, region, pollInterval, callback) {
  if (typeof pollInterval === 'function') {
    callback = pollInterval;
    pollInterval = 10000;
  }

  var cfn = new AWS.CloudFormation({ region: region });

  var events = eventStream(cfn, name, { pollInterval: pollInterval })
    .on('error', function(err) {
      return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    });

  var stringify = new stream.Transform({ objectMode: true });
  stringify._transform = function(event, enc, callback) {
    var msg = event.ResourceStatus[colors[event.ResourceStatus]] + ' ' + event.LogicalResourceId;
    if (event.ResourceStatusReason) msg += ': ' + event.ResourceStatusReason;
    callback(null, currentTime() + ' ' + region + ': ' + msg + '\n');
  };

  events.pipe(stringify).pipe(process.stdout);
  stringify.on('end', callback);
};

/**
 * Validate a CloudFormation template
 *
 * @param {string} region - the region the stack would run in
 * @param {string} templateUrl - the URL for the template on S3
 * @param {function} callback - a function called when the template has been validated.
 * An invalid template will result in an error being provided to the callback. No error
 * object indicates that the template appears valid.
 */
actions.validate = function(region, templateUrl, callback) {
  var cfn = new AWS.CloudFormation({ region: region });

  cfn.validateTemplate({ TemplateURL: templateUrl }, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    callback();
  });
};

/**
 * Save a CloudFormation stack's configuration to a local file or S3
 *
 * @param {string} name - the base name of the stack (no suffix)
 * @param {string} bucket - the name of the S3 bucket to save the configuration into
 * @param {string} config - the name of the saved configuration
 * @param {object} parameters - name/value pairs defining the stack configuration to save
 * @param {string} [kms] - if desired, the ID of the AWS KMS master encryption key to use
 * to encrypt this configuration at rest
 * @param {function} callback - a function that will be called when the configuration is saved
 */
actions.saveConfiguration = function(name, bucket, config, parameters, kms, callback) {
  if (typeof kms === 'function') {
    callback = kms;
    kms = null;
  }

  lookup.bucketRegion(bucket, function(err, region) {
    var s3 = new AWS.S3({ region: region });
    var params = {
      Bucket: bucket,
      Key: lookup.configKey(name, config),
      Body: JSON.stringify(parameters)
    };

    if (kms) {
      params.ServerSideEncryption = 'aws:kms';
      params.SSEKMSKeyId = kms;
    }

    s3.putObject(params, function(err) {
      if (err && err.code === 'NoSuchBucket')
        return callback(new actions.BucketNotFoundError('S3 bucket %s not found in %s', bucket, region));
      if (err)
        return callback(new actions.S3Error('%s: %s', err.code, err.message));

      callback();
    });
  });
};

/**
 * Save a CloudFormation template to S3
 *
 * @param {string} templateUrl - an S3 URL where the template will be saved
 * @param {string} templateBody - the CloudFormation template as a JSON string
 * @param {function} callback - a function fired when the template has been saved
 */
actions.saveTemplate = function(templateUrl, templateBody, callback) {
  var uri = url.parse(templateUrl);
  var prefix = uri.host.replace(/^s3[-\.]/, '').split('.');
  var region = prefix.length === 4 ? 'cn-north-1' :
    prefix.length === 2 ? 'us-east-1' : prefix[0];

  var s3 = new AWS.S3({ region: region });
  var params = Object.assign({
    Body: templateBody
  }, s3urls.fromUrl(templateUrl));

  s3.putObject(params, function(err) {
    if (err && err.code === 'NoSuchBucket')
      return callback(new actions.BucketNotFoundError('S3 bucket %s not found in %s', params.Bucket, region));
    if (err)
      return callback(new actions.S3Error('%s: %s', err.code, err.message));

    callback();
  });
};

/**
 * Create an S3 URL for a template
 *
 * @param {string} bucket - the bucket in which the template will be placed
 * @param {string} region - the region that the bucket is in
 * @param {string} name - the base name of the stack
 * @returns {string} an S3 URL where the template will be saved
 */
actions.templateUrl = function(bucket, region, name) {
  var key = cuid() + '-' + name + '.template.json';

  var host;
  if (region == 'us-east-1') {
    host = 'https://s3.amazonaws.com';
  } else if (region.match(/^cn-/)) {
    host = 'https://s3.' + region + '.amazonaws.com.cn';
  } else {
    host = 'https://s3-' + region + '.amazonaws.com';
  }

  return [host, bucket, key].join('/');
};

/**
 * Error representing an unexpected failure in a CloudFormation request
 */
actions.CloudFormationError = error('CloudFormationError');

/**
 * Error representing a bucket that does not exist
 */
actions.BucketNotFoundError = error('BucketNotFoundError');

/**
 * Error representing an unexpected failure in an S3 request
 */
actions.S3Error = error('S3Error');

/**
 * Error representing an attempt to execute a changeset that is not executable
 */
actions.ChangeSetNotExecutableError = error('ChangeSetNotExecutableError');

/**
 * Poll changeset until it has reached a completed state.
 *
 * @private
 * @param {object} cfn - a cloudformation client object
 * @param {string} name - the name of the existing stack to update
 * @param {string} changesetId - the name or ARN of an existing changeset
 * @param {function} callback - a function that will be provided with the changeset
 * details once the changeset is in CREATE_COMPLETE, DELETE_COMPLETE, or FAILED state
 */
function describeChangeset(cfn, name, changesetId, callback) {
  cfn.describeChangeSet({ ChangeSetName: changesetId, StackName: name }, function(err, data) {
    if (err) return callback(err);

    if (data.Status === 'CREATE_COMPLETE' || data.Status === 'FAILED' || data.status === 'DELETE_COMPLETE')
      return callback(null, data);

    setTimeout(describeChangeset, 1000, cfn, name, changesetId, callback);
  });
}

/**
 * Build parameters object for CloudFormation requests
 *
 * @private
 * @param {string} name - the name of the stack
 * @param {string} templateUrl - the URL for the template on S3
 * @param {object} parameters - name/value pairs defining the desired stack configuration
 * @returns {object} params - an object for use in create/update/delete stack requests
 */
function stackParameters(name, templateUrl, parameters) {
  return {
    StackName: name,
    Capabilities: ['CAPABILITY_IAM'],
    TemplateURL: templateUrl,
    Parameters: Object.keys(parameters).map(function(key) {
      return { ParameterKey: key, ParameterValue: parameters[key] };
    })
  };
}

function currentTime() {
  var now = new Date();
  var hour = ('00' + now.getUTCHours()).slice(-2);
  var min = ('00' + now.getUTCMinutes()).slice(-2);
  var sec = ('00' + now.getUTCSeconds()).slice(-2);
  return [hour, min, sec].join(':') + 'Z';
}

==> cfn-config-2.3.0.tgz: package/lib/actions.js <==
var url = require('url');
var crypto = require('crypto');
var stream = require('stream');
var AWS = require('aws-sdk');
var cuid = require('cuid');
var error = require('fasterror');
var s3urls = require('s3urls');
var eventStream = require('cfn-stack-event-stream');
var lookup = require('./lookup');

require('colors');

var colors = {
  CREATE_IN_PROGRESS: 'yellow',
  CREATE_FAILED: 'red',
  CREATE_COMPLETE: 'green',
  DELETE_IN_PROGRESS: 'yellow',
  DELETE_FAILED: 'red',
  DELETE_COMPLETE: 'grey',
  UPDATE_IN_PROGRESS: 'yellow',
  UPDATE_COMPLETE_CLEANUP_IN_PROGRESS: 'yellow',
  UPDATE_FAILED: 'red',
  UPDATE_COMPLETE: 'green',
  ROLLBACK_IN_PROGRESS: 'red',
  ROLLBACK_COMPLETE: 'red',
  ROLLBACK_FAILED: 'red',
  UPDATE_ROLLBACK_FAILED: 'red',
  UPDATE_ROLLBACK_COMPLETE_CLEANUP_IN_PROGRESS: 'yellow',
  UPDATE_ROLLBACK_IN_PROGRESS: 'yellow'
};

var actions = module.exports = {};

/**
 * Create a new CloudFormation stack
 *
 * @param {string} name - the name of the new stack
 * @param {string} region - the region to launch the stack into
 * @param {string} templateUrl - the URL for the template on S3
 * @param {object} parameters - name/value pairs defining the desired stack configuration
 * @param {function} callback - a function called when stack creation is complete
 */
actions.create = function(name, region, templateUrl, parameters, callback) {
  var cfn = new AWS.CloudFormation({ region: region });
  var params = stackParameters(name, templateUrl, parameters);

  cfn.createStack(params, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    callback();
  });
};

/**
 * Update an existing CloudFormation stack
 *
 * @param {string} name - the name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {string} templateUrl - the URL for the template on S3
 * @param {object} parameters - name/value pairs defining the desired stack configuration
 * @param {function} callback - a function called when the stack update has been invoked
 */
actions.update = function(name, region, templateUrl, parameters, callback) {
  var cfn = new AWS.CloudFormation({ region: region });
  var params = stackParameters(name, templateUrl, parameters);

  cfn.updateStack(params, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    callback();
  });
};

/**
 * Determine what will change about an existing CloudFormation stack by
 * performing a specific update
 *
 * @param {string} name - the name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {string} templateUrl - the URL for the template on S3
 * @param {object} parameters - name/value pairs defining the desired stack configuration
 * @param {function} callback - a function provided with a summary of the changes that
 * this change will perform to stack resources
 */
actions.diff = function(name, region, templateUrl, parameters, callback) {
  var cfn = new AWS.CloudFormation({ region: region });
  var changesetId = 'a' + crypto.randomBytes(16).toString('hex');
  var params = stackParameters(name, templateUrl, parameters);
  params.ChangeSetName = changesetId;

  cfn.createChangeSet(params, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));

    describeChangeset(cfn, name, changesetId, function(err, data) {
      if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));

      var details = {
        id: data.ChangeSetName,
        status: data.Status,
        execution: data.ExecutionStatus
      };

      if (data.Changes) details.changes = data.Changes.map(function(change) {
        return {
          id: change.ResourceChange.PhysicalResourceId,
          name: change.ResourceChange.LogicalResourceId,
          type: change.ResourceChange.ResourceType,
          action: change.ResourceChange.Action,
          replacement: change.ResourceChange.Replacement === 'True'
        };
      });

      callback(null, details);
    });
  });
};

/**
 * Execute a ChangeSet in order to perform an update on an existing CloudFormation stack
 *
 * @param {string} name - the name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {string} changesetId - the name or ARN of an existing changeset
 * @param {function} callback - a function called when the stack update has been invoked
 */
actions.executeChangeSet = function(name, region, changesetId, callback) {
  var cfn = new AWS.CloudFormation({ region: region });

  describeChangeset(cfn, name, changesetId, function(err, data) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));

    if (data.ExecutionStatus !== 'AVAILABLE') {
      err = new actions.ChangeSetNotExecutableError('Cannot execute changeset');
      err.execution = data.ExecutionStatus;
      err.status = data.Status;
      err.reason = data.StatusReason;
      return callback(err);
    }

    cfn.executeChangeSet({ StackName: name, ChangeSetName: changesetId }, function(err) {
      if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
      callback();
    });
  });
};

/**
 * Delete an existing CloudFormation stack
 *
 * @param {string} name - the name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {function} callback - a function called when stack deletion is complete
 */
actions.delete = function(name, region, callback) {
  var cfn = new AWS.CloudFormation({ region: region });
  cfn.deleteStack({ StackName: name }, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    callback();
  });
};

/**
 * Monitor a stack throughout a create, delete, or update
 *
 * @param {string} name - the full name of the existing stack to update
 * @param {string} region - the region the stack is in
 * @param {number} [pollInterval=10000] - ms to wait between requests for
 * CloudFormation stack events
 * @param {function} callback - a function called when stack action is complete
 */
actions.monitor = function(name, region, pollInterval, callback) {
  if (typeof pollInterval === 'function') {
    callback = pollInterval;
    pollInterval = 10000;
  }

  var cfn = new AWS.CloudFormation({ region: region });

  var events = eventStream(cfn, name, { pollInterval: pollInterval })
    .on('error', function(err) {
      return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    });

  var stringify = new stream.Transform({ objectMode: true });
  stringify._transform = function(event, enc, callback) {
    var msg = event.ResourceStatus[colors[event.ResourceStatus]] + ' ' + event.LogicalResourceId;
    if (event.ResourceStatusReason) msg += ': ' + event.ResourceStatusReason;
    callback(null, currentTime() + ' ' + region + ': ' + msg + '\n');
  };

  events.pipe(stringify).pipe(process.stdout);
  stringify.on('end', callback);
};

/**
 * Validate a CloudFormation template
 *
 * @param {string} region - the region the stack would run in
 * @param {string} templateUrl - the URL for the template on S3
 * @param {function} callback - a function called when the template has been validated.
 * An invalid template will result in an error being provided to the callback. No error
 * object indicates that the template appears valid.
 */
actions.validate = function(region, templateUrl, callback) {
  var cfn = new AWS.CloudFormation({ region: region });

  cfn.validateTemplate({ TemplateURL: templateUrl }, function(err) {
    if (err) return callback(new actions.CloudFormationError('%s: %s', err.code, err.message));
    callback();
  });
};

/**
 * Save a CloudFormation stack's configuration to a local file or S3
 *
 * @param {string} name - the base name of the stack (no suffix)
 * @param {string} bucket - the name of the S3 bucket to save the configuration into
 * @param {string} config - the name of the saved configuration
 * @param {object} parameters - name/value pairs defining the stack configuration to save
 * @param {string} [kms] - if desired, the ID of the AWS KMS master encryption key to use
 * to encrypt this configuration at rest
 * @param {function} callback - a function that will be called when the configuration is saved
 */
actions.saveConfiguration = function(name, bucket, config, parameters, kms, callback) {
  if (typeof kms === 'function') {
    callback = kms;
    kms = null;
  }

  lookup.bucketRegion(bucket, function(err, region) {
    var s3 = new AWS.S3({ region: region });
    var params = {
      Bucket: bucket,
      Key: lookup.configKey(name, config),
      Body: JSON.stringify(parameters)
    };

    if (kms) {
      params.ServerSideEncryption = 'aws:kms';
      params.SSEKMSKeyId = kms;
    }

    s3.putObject(params, function(err) {
      if (err && err.code === 'NoSuchBucket')
        return callback(new actions.BucketNotFoundError('S3 bucket %s not found in %s', bucket, region));
      if (err)
        return callback(new actions.S3Error('%s: %s', err.code, err.message));

      callback();
    });
  });
};

/**
 * Save a CloudFormation template to S3
 *
 * @param {string} templateUrl - an S3 URL where the template will be saved
 * @param {string} templateBody - the CloudFormation template as a JSON string
 * @param {function} callback - a function fired when the template has been saved
 */
actions.saveTemplate = function(templateUrl, templateBody, callback) {
  var uri = url.parse(templateUrl);
  var prefix = uri.host.replace(/^s3[-\.]/, '').split('.');
  var region = prefix.length === 4 ? 'cn-north-1' :
    prefix.length === 2 ? 'us-east-1' : prefix[0];

  var s3 = new AWS.S3({ region: region });
  var params = Object.assign({
    Body: templateBody
  }, s3urls.fromUrl(templateUrl));

  s3.putObject(params, function(err) {
    if (err && err.code === 'NoSuchBucket')
      return callback(new actions.BucketNotFoundError('S3 bucket %s not found in %s', params.Bucket, region));
    if (err)
      return callback(new actions.S3Error('%s: %s', err.code, err.message));

    callback();
  });
};

/**
 * Create an S3 URL for a template
 *
 * @param {string} bucket - the bucket in which the template will be placed
 * @param {string} region - the region that the bucket is in
 * @param {string} name - the base name of the stack
 * @returns {string} an S3 URL where the template will be saved
 */
actions.templateUrl = function(bucket, region, name) {
  var key = cuid() + '-' + name + '.template.json';

  var host;
  if (region == 'us-east-1') {
    host = 'https://s3.amazonaws.com';
  } else if (region.match(/^cn-/)) {
    host = 'https://s3.' + region + '.amazonaws.com.cn';
  } else {
    host = 'https://s3-' + region + '.amazonaws.com';
  }

  return [host, bucket, key].join('/');
};

/**
 * Error representing an unexpected failure in a CloudFormation request
 */
actions.CloudFormationError = error('CloudFormationError');

/**
 * Error representing a bucket that does not exist
 */
actions.BucketNotFoundError = error('BucketNotFoundError');

/**
 * Error representing an unexpected failure in an S3 request
 */
actions.S3Error = error('S3Error');

/**
 * Error representing an attempt to execute a changeset that is not executable
 */
actions.ChangeSetNotExecutableError = error('ChangeSetNotExecutableError');

/**
 * Poll changeset until it has reached a completed state.
 *
 * @private
 * @param {object} cfn - a cloudformation client object
 * @param {string} name - the name of the existing stack to update
 * @param {string} changesetId - the name or ARN of an existing changeset
 * @param {function} callback - a function that will be provided with the changeset
 * details once the changeset is in CREATE_COMPLETE, DELETE_COMPLETE, or FAILED state
 */
function describeChangeset(cfn, name, changesetId, callback) {
  cfn.describeChangeSet({ ChangeSetName: changesetId, StackName: name }, function(err, data) {
    if (err) return callback(err);

    if (data.Status === 'CREATE_COMPLETE' || data.Status === 'FAILED' || data.status === 'DELETE_COMPLETE')
      return callback(null, data);

    setTimeout(describeChangeset, 1000, cfn, name, changesetId, callback);
  });
}

/**
 * Build parameters object for CloudFormation requests
 *
 * @private
 * @param {string} name - the name of the stack
 * @param {string} templateUrl - the URL for the template on S3
 * @param {object} parameters - name/value pairs defining the desired stack configuration
 * @returns {object} params - an object for use in create/update/delete stack requests
 */
function stackParameters(name, templateUrl, parameters) {
  return {
    StackName: name,
    Capabilities: ['CAPABILITY_IAM'],
    TemplateURL: templateUrl,
    Parameters: Object.keys(parameters).map(function(key) {
      return { ParameterKey: key, ParameterValue: parameters[key] };
    })
  };
}

function currentTime() {
  var now = new Date();
  var hour = ('00' + now.getUTCHours()).slice(-2);
  var min = ('00' + now.getUTCMinutes()).slice(-2);
  var sec = ('00' + now.getUTCSeconds()).slice(-2);
  return [hour, min, sec].join(':') + 'Z';
}
